/**
 * QApplication wrapper that can act as a Sugar activity
 *
 * (C) 2007 Bernhard Rosenkraenzer <bero@arklinux.org>
 *
 * Released under the terms of the GNU General Public License (GPL)
 * version 2, or, at your opinion, any later version of the GPL
 * published by the Free Software Foundation.
 *
 * For alternative licensing options, contact bero@arklinux.org
 */
#include "QActivity.moc"
#include <QString>
#include <QX11Info>
#include <QDBusConnection>
#include <QDesktopWidget>
#include <QFile>
#include <cstring>
#include <iostream>

extern "C" {
#include <X11/Xlib.h>
#include <sys/syslog.h>
#include <sys/stat.h>
#include <stdlib.h>
}

using namespace std;

QActivity::QActivity(int &argc, char **argv):QApplication(argc, argv),_toplevel(0),_dbus(0)
{
	// According to http://wiki.laptop.org/go/Activity_DBus_API, we have to watch
	// out for 4 extra command line switches...
	// The - and -- variants are from the Activity_DBus_API, the other one is what
	// I've actually observed sugar passes along -- leaving all in place for the
	// time being, just in case which variant you need depends on the version of sugar.
	for(int i=0; i<argc; ++i) {
		if(i+1<argc) {
			if(!strcmp(argv[i], "-b") || !strcmp(argv[i], "--bundle-id") || !strcmp(argv[i], "bundle_id"))
				_bundleId=argv[++i];
			else if(!strcmp(argv[i], "-a") || !strcmp(argv[i], "--activity-id") || !strcmp(argv[i], "activity_id"))
				_activityId=argv[++i];
			else if(!strcmp(argv[i], "-o") || !strcmp(argv[i], "--object-id") || !strcmp(argv[i], "object_id"))
				_objectId=argv[++i];
			else if(!strcmp(argv[i], "-u") || !strcmp(argv[i], "--uri") || !strcmp(argv[i], "uri"))
				_uri=argv[++i];
		}
	}

	if(!_activityId.isEmpty()) {
		// if it's empty, we aren't in sugar and don't need to bother
		_dbus=new SugarDBusAdaptor(this);
		QDBusConnection sessionBus=QDBusConnection::sessionBus();
		if(!sessionBus.registerService("org.laptop.Activity" + _activityId))
			syslog(LOG_ERR, "Ewkat: failed to register d-bus service");
		if(!sessionBus.registerObject("/org/laptop/Activity/" + _activityId, _dbus, QDBusConnection::ExportAdaptors))
			syslog(LOG_ERR, "Ewkat: failed to register d-bus object");
		connect(_dbus, SIGNAL(SetActive(bool)), this, SIGNAL(setActive(bool)));
		connect(_dbus, SIGNAL(takeScreenshot()), this, SIGNAL(takeScreenshot()));
		connect(_dbus, SIGNAL(invite(QString)), this, SIGNAL(invite(QString)));
	}

	if(getenv("SUGAR_ACTIVITY_ROOT")) {
		// Use sugar style directories
		_dataDir=QString(getenv("SUGAR_ACTIVITY_ROOT")) + "/data";
		_instanceDir=QString(getenv("SUGAR_ACTIVITY_ROOT")) + "/instance";
		_tmpDir=QString(getenv("SUGAR_ACTIVITY_ROOT")) + "/tmp";
	} else {
		// Create work directories without sugar
		_dataDir=DATADIR "/data";
		char tmp[128];
		strcpy(tmp, DATADIR "/instanceXXXXXX");
		_instanceDir=mkdtemp(tmp);
		strcpy(tmp, DATADIR "/tmpXXXXXX");
		_tmpDir=mkdtemp(tmp);
	}
}

QActivity::~QActivity()
{
	// if we aren't in sugar, we need to clean up the instance dirs...
	if(!getenv("SUGAR_ACTIVITY_ROOT")) {
		system(QFile::encodeName("rm -rf " + _instanceDir).data());
		system(QFile::encodeName("rm -rf " + _tmpDir).data());
	}
}

bool QActivity::inSugar() const
{
	return _dbus != 0;
}

void QActivity::setTopLevelWidget(QWidget *w)
{
	_toplevel=w;
	w->setGeometry(0, 0, desktop()->width(), desktop()->height());

	if(_activityId.isEmpty()) {
		// We aren't running in sugar - no need to bother setting up
		// integration
		return;
	}
	// According to http://wiki.laptop.org/go/Activity_DBus_API, we have to set
	// these properties on the top-level window:
	// _SUGAR_BUNDLE_ID - bundle ID
	// _SUGAR_ACTIVITY_ID - activity ID
	// _NET_WM_NAME - activity title [As it is a general standard, Qt sets
	//                this by itself, we don't need to bother]
	// _NET_WM_PID - process ID [As it is a general standard, Qt sets this
	//               by itself, we don't need to bother]
	Display * const dpy=QX11Info::display();
	Atom _SUGAR_BUNDLE_ID=XInternAtom(dpy, (char*)"_SUGAR_BUNDLE_ID\0", False);
	Atom _SUGAR_ACTIVITY_ID=XInternAtom(dpy, (char*)"_SUGAR_ACTIVITY_ID\0", False);
	Atom STRING=XInternAtom(dpy, (char*)"STRING\0", False);

	XChangeProperty(dpy, _toplevel->winId(), _SUGAR_BUNDLE_ID, STRING, 8, PropModeReplace, (unsigned char *)_bundleId.toLatin1().data(), _bundleId.toLatin1().size());
	XChangeProperty(dpy, _toplevel->winId(), _SUGAR_ACTIVITY_ID, STRING, 8, PropModeReplace, (unsigned char *)_activityId.toLatin1().data(), _activityId.toLatin1().size());
}
